import os
import sqlite3
import shutil
from flask import Blueprint, request
import configparser
from a2d.routes.auth import login_required
from a2d.modals.user import decrypt_user_passphrase
from a2d.a2d_utils.a2d_utils import remove_cronjob
from a2d.routes.dns import update_nginx_config
from a2d.routes.certs import a2d_rm_selfssl, a2d_ca_list, a2d_rm_cassl, reload_nginx, enable_default_ng

reset_routes = Blueprint('reset', __name__)

adv_conf = '/etc/a2d/a2d_adv_conf.ini'
dapnet_data = '/etc/a2d/dapnet_data.ini'
runscript_data = '/etc/a2d/runscript_data.ini'

@reset_routes.route('/reset-portal', methods=['POST'])
@login_required
def reset_account():
    passphrase = request.form.get("passphrase")
    if not verify_passphrase(passphrase):
        return "Invalid Passphrase", 400
    
    reset_confirm = request.form['reset_confirm']
    selfSSL_delete = request.form['self_ssl_delete'] == 'true'
    caSSL_delete = request.form['ca_ssl_delete'] == 'true'
    
    if reset_confirm.lower() == 'delete':
        remove_cronjob()
        rm_dbs()
        
        #a2d_adv_conf.ini
        config1 = configparser.ConfigParser()
        config1.optionxform = lambda option: option  # Preserve case sensitivity
        config1.read(adv_conf)
        config1.set('Timer', 'IntervalMinutes', '15')
        config1.set('Timer', 'IntervalHours', '0')
        config1.set('SSID', 'PreferredSSIDs', '1,2,3,4,5,6,7,8,9,10,11,12,13,14,15')
        config1.set('LOGS', 'NumberOfLogs', '150')
        with open(adv_conf, 'w') as config_file1:
            config1.write(config_file1, space_around_delimiters=False)
        
        #dapnet_data.ini
        config2 = configparser.ConfigParser()
        config2.optionxform = lambda option: option  # Preserve case sensitivity
        config2.read(dapnet_data)
        config2.set('DAPNETVerify', 'HTTPStatus', '201')
        with open(dapnet_data, 'w') as config_file2:
            config2.write(config_file2, space_around_delimiters=False)
        
        #runscript_data.ini
        config3 = configparser.ConfigParser()
        config3.optionxform = lambda option: option  # Preserve case sensitivity
        config3.read(runscript_data)
        config3.set('InternalTimeStamp', 'LastRunTime', '')
        config3.set('APRSKeyVerify', 'apiKeyStatus', 'valid')
        config3.set('APRSTimeStamp', 'LastAPRSPull', '')
        config3.set('APRSCheck', 'APRSCheck', 'verified')
        config3.set('APRSCheck', 'LastCheck', '')
        with open(runscript_data, 'w') as config_file3:
            config3.write(config_file3, space_around_delimiters=False)
        
        if selfSSL_delete:
            a2d_rm_selfssl()
            a2d_default_dns()

        if caSSL_delete:
            cassl_certs_list = a2d_ca_list()
            for rm_cassl_certs in cassl_certs_list:
                a2d_rm_cassl(rm_cassl_certs)
            a2d_default_dns()

        try:
            #Remove all creds and keys
            shutil.rmtree('/etc/a2d/.creds')
            shutil.rmtree('/etc/a2d/.keys')
        except Exception as e:
            pass
        
        return 'logout'
    else:
        return 'Type delete'

def verify_passphrase(passphrase):
    stored_passphrase = decrypt_user_passphrase()
    return passphrase == stored_passphrase

def a2d_default_dns():
    listen_port = '9331'
    set_selfssl_status = 'disable'
    server_name = '_'
    set_cassl_status = ''
    update_nginx_config(listen_port, server_name, set_selfssl_status, set_cassl_status)
    enable_default_ng()
    reload_nginx()

def rm_dbs():
    db_folder = '/var/lib/a2d/dbs'
    aprs_db_path = os.path.join(db_folder, 'aprs_messages.db')
    pseudo_aprs_db_path = os.path.join(db_folder, 'pseudo_aprs_messages.db')
    callssid_db_path = os.path.join(db_folder, 'callssid.db')
    ham_info_db_path = os.path.join(db_folder, 'ham_info.db')
    previous_values_db_path = os.path.join(db_folder, 'previous_values.db')

    try:
        clear_table(aprs_db_path, 'aprs_messages')
        clear_table(pseudo_aprs_db_path, 'pseudo_aprs_messages')
        clear_table(callssid_db_path, 'callssid')
        clear_table(ham_info_db_path, 'ham_info')
        clear_table(previous_values_db_path, 'previous_values')
    except Exception as e:
        pass

def clear_table(db_path, table_name):
    try:
        conn = sqlite3.connect(db_path)
        cursor = conn.cursor()
        
        cursor.execute(f'DELETE FROM {table_name}')

        conn.commit()
        conn.close()

        os.remove(db_path)

    except Exception as e:
        pass



#coding=utf-8
#!/usr/bin/env python3

import configparser
import multiprocessing
from a2d.a2d_utils.decr_usrinfo import retrieve_usrinfo
from a2d.a2d_utils.a2d_utils import establish_connection, create_table_if_not_exists, retrieve_data_from_table

def update_previous_values_db(previous_values_db, callsign_nossid, txgroup, condition1, condition2):
    conn, cursor = establish_connection(previous_values_db)

    create_table_if_not_exists(cursor, "previous_values", "(id INTEGER PRIMARY KEY, callsign TEXT, txgroup TEXT)")

    cursor.execute('SELECT callsign, txgroup FROM previous_values WHERE id = 1')
    row = cursor.fetchone()

    if row is None or row[0] != callsign_nossid:
        cursor.execute('REPLACE INTO previous_values (id, callsign, txgroup) VALUES (1, ?, ?)', (callsign_nossid, txgroup))
        conn.commit()
        condition1 = True
    elif row[1] != txgroup:
        cursor.execute('REPLACE INTO previous_values (id, callsign, txgroup) VALUES (1, ?, ?)', (callsign_nossid, txgroup))
        conn.commit()
        condition2 = True
    else:
        condition1 = False
        condition2 = False

    conn.close()
    return condition1, condition2

def update_callssid_db(callssid_db, callsign_nossid, ssids, condition1):
    conn, cursor = establish_connection(callssid_db)

    create_table_if_not_exists(cursor, "callssid", "(id INTEGER PRIMARY KEY, callsign TEXT, ssid TEXT)")

    callssid_conn, callssid_cursor = establish_connection(callssid_db)
    rows = retrieve_data_from_table(callssid_cursor, 'callssid')
    callssid_conn.close()

    # Get the SSIDs from the callssid database
    existing_ssids = [row[2] for row in rows if row[1] == callsign_nossid]

    if condition1:
        cursor.execute("DELETE FROM callssid")

    if ssids != existing_ssids:  # Compare SSIDs from the database with those in a2d_adv_conf.ini
        cursor.execute("DELETE FROM callssid WHERE callsign = ?", (callsign_nossid,))
        data = [(callsign_nossid, ssid.strip()) for ssid in ssids]
        cursor.executemany('INSERT OR REPLACE INTO callssid (callsign, ssid) VALUES (?, ?)', data)

    conn.commit()
    conn.close()
    pass

def update_ham_info_db(ham_info_db, callsign_nossid, txgroup, condition1, condition2):
    conn, cursor = establish_connection(ham_info_db)

    create_table_if_not_exists(cursor, "ham_info", "(id INTEGER PRIMARY KEY, callsign TEXT, txgroup TEXT)")

    if condition1 or condition2:
        cursor.execute('''
            REPLACE INTO ham_info VALUES
            (?, ?, ?)''', (1, callsign_nossid, txgroup)
        )
        conn.commit()
    conn.close()
    pass

if __name__ == "__main__":
    data = retrieve_usrinfo()
    callsign_nossid = data['callsign_nossid']
    txgroup = data['txgroup']

    config = configparser.ConfigParser()
    config.read("/etc/a2d/a2d_adv_conf.ini")
    preferred_ssids = config.get("SSID", "PreferredSSIDs", fallback="")
    ssids = preferred_ssids.split(",")

    condition1 = False
    condition2 = False

    data_dir = '/var/lib/a2d/dbs'
    previous_values_db = f'{data_dir}/previous_values.db'
    callssid_db = f'{data_dir}/callssid.db'
    ham_info_db = f'{data_dir}/ham_info.db'

    # Call the database update functions
    condition1, condition2 = update_previous_values_db(previous_values_db, callsign_nossid, txgroup, condition1, condition2)
    
    num_processes = multiprocessing.cpu_count()
    pool = multiprocessing.Pool(processes=num_processes)
    
    result1 = pool.apply_async(update_callssid_db, (callssid_db, callsign_nossid, ssids, condition1))
    result2 = pool.apply_async(update_ham_info_db, (ham_info_db, callsign_nossid, txgroup, condition1, condition2))
    
    pool.close()
    pool.join()



import subprocess
import re

NGINX_CONFIG_PATH = '/etc/nginx/conf.d/00-a2d.conf'
CORE_NGINX_CONF = '/etc/nginx/nginx.conf'

def reload_nginx():
    try:
        subprocess.run(["systemctl", "reload", "nginx"], check=True)
    except subprocess.CalledProcessError as e:
        return "Error reloading nginx:", e

def read_nginx_config(key):
    try:
        with open(NGINX_CONFIG_PATH, 'r') as config_file:
            for line in config_file:
                if line.strip().startswith(f"{key} "):
                    return line.split()[-1].rstrip(';')
    except Exception as e:
        return f"Error reading nginx conf: {e}"

def disable_default_ng():
    with open(CORE_NGINX_CONF, 'r') as nginx_conf_file:
        nginx_conf_content = nginx_conf_file.readlines()

    # Search for the line with 'include /etc/nginx/sites-enabled/*;' and comment it out
    for i, line in enumerate(nginx_conf_content):
        if 'include /etc/nginx/sites-enabled/*;' in line:
            if not line.startswith('#'):
                nginx_conf_content[i] = '#' + line

    with open(CORE_NGINX_CONF, 'w') as nginx_conf_file:
        nginx_conf_file.writelines(nginx_conf_content)

def enable_default_ng():
    with open(CORE_NGINX_CONF, 'r') as nginx_conf_file:
        nginx_conf_content = nginx_conf_file.readlines()

    # Search for the line with 'include /etc/nginx/sites-enabled/*;' and uncomment it
    for i, line in enumerate(nginx_conf_content):
        if re.match(r'^\s*#\s*include /etc/nginx/sites-enabled/\*;', line):
            nginx_conf_content[i] = re.sub(r'^\s*#\s*', '', line)

    with open(CORE_NGINX_CONF, 'w') as nginx_conf_file:
        nginx_conf_file.writelines(nginx_conf_content)



#coding=utf-8
#!/usr/bin/env python3

import os
import subprocess
import time
from a2d.a2d_utils.get_aprs import aprs_check
from a2d.a2d_utils.a2d_utils import remove_cronjob, read_ini_data, write_ini_data

runscript_data = '/etc/a2d/runscript_data.ini'

def run_script():
    config = read_ini_data(runscript_data)
    current_time = time.time()
    str_current_time = str(current_time)

    #Check when was the last APRS Check
    last_aprs_check_ini = config.get('APRSCheck', 'LastCheck', fallback='')
    if last_aprs_check_ini:
        last_aprs_check_ini = float(last_aprs_check_ini)
        last_check_interval = current_time - last_aprs_check_ini

        if last_check_interval >= 60 * 3:
            config.set('APRSCheck', 'APRSCheck', 'unverified')
            write_ini_data(config, runscript_data)

    aprs_check_ini = config.get('APRSCheck', 'APRSCheck', fallback='')
    if aprs_check_ini == 'unverified':
        config.set('APRSCheck', 'LastCheck', str_current_time)

        # Check the APRS API response for the current callsign and API key
        entries_check = aprs_check()

        if entries_check is not None and "code" in entries_check:
            code = entries_check["code"]
            if code == "apikey-wrong":
                # Write the key status in /etc/a2d/runscript_data.ini
                config.set('APRSKeyVerify', 'apiKeyStatus', 'wrong')
                write_ini_data(config, runscript_data)
                
                # Terminate the script if the API key is wrong
                remove_cronjob()
                return

            elif code == "apikey-invalid":
                # Write the key status in /etc/a2d/runscript_data.ini
                config.set('APRSKeyVerify', 'apiKeyStatus', 'invalid')
                write_ini_data(config, runscript_data)

                # Terminate the script if the API key is invalid
                remove_cronjob()
                return

        config.set('APRSKeyVerify', 'apiKeyStatus', 'valid')
        config.set('APRSCheck', 'APRSCheck', 'verified')
        write_ini_data(config, runscript_data)

    # Continue running the script to restrict aprs pull
    last_aprs_pull = config.get('APRSTimeStamp', 'LastAPRSPull', fallback='')
    if last_aprs_pull:
        last_aprs_pull = float(last_aprs_pull)
        time_difference = current_time - last_aprs_pull
        ideal_aprs_interval = 121
    
        if time_difference <= ideal_aprs_interval:
            time_to_sleep = ideal_aprs_interval - time_difference
            time.sleep(time_to_sleep)

    # Continue running the script and write the start time
    #Write start time
    config.set('InternalTimeStamp', 'LastRunTime', str_current_time)
    write_ini_data(config, runscript_data)

    db_file_path = "/var/lib/a2d/dbs/aprs_messages.db"
    if os.path.exists(db_file_path):
        modules = ["a2d.build_usrdbs", "a2d.push_a2d"]
    else:
        modules = ["a2d.build_usrdbs", "a2d.pseudorun"]

    for module in modules:
        subprocess.call(["python3", "-m", module])

    str_current_time2 = str(time.time())
    config.set('APRSTimeStamp', 'LastAPRSPull', str_current_time2)
    write_ini_data(config, runscript_data)

if __name__ == '__main__':
    run_script()



import os
import subprocess
import re
from datetime import datetime
from flask import Blueprint, request, render_template, jsonify
from a2d.modals.creds import get_credentials
from a2d.routes.auth import login_required
from a2d.routes.certs import read_ssl, a2d_self_ssl, a2d_rm_cassl, a2d_ca_ssl, a2d_ca_list
from a2d.routes.nginx import reload_nginx, read_nginx_config, disable_default_ng, enable_default_ng

dns_routes = Blueprint('dns', __name__)

# Define the directory paths
NGINX_CONFIG_PATH = '/etc/nginx/conf.d/00-a2d.conf'
SSL_CERT_PATH = '/etc/nginx/ssl/a2d-ssl.crt'
SSL_KEY_PATH = '/etc/nginx/ssl/a2d-ssl.key'

#DNS data to HTML
@dns_routes.route('/local-dns-setting', methods=['GET'])
@login_required
def local_dns_setting():
    creds = get_credentials()  # Need this to display Navbar_full in dns.html
    
    #SSL status and server name
    current_listen_port = read_nginx_config('listen')
    if current_listen_port == 'ssl': #Strip from nginx conifg file
        current_listen_port = '443'
    current_server_name = read_nginx_config('server_name')
    current_ssl_status = current_listen_port == '443'
    a2d_default_dns = (current_listen_port == '9331' and current_server_name == '_' and not current_ssl_status)

    #Self SSL details
    output = read_ssl(SSL_CERT_PATH)
    stored_selfcommon_name, stored_selforganization_name, _ = output.split(',')
    if stored_selfcommon_name == '':
        stored_selfcommon_name = 'None'
    if stored_selforganization_name == '':
        stored_selforganization_name = 'None'

    #Get cassl list
    cassl_certs_list = list_cassl_certs()

    return render_template(
        'dns.html', 
        listen_port=current_listen_port, 
        server_name=current_server_name, 
        current_ssl_status=current_ssl_status, 
        a2d_default_dns=a2d_default_dns, 
        stored_selfcommon_name=stored_selfcommon_name, 
        stored_selforganization_name=stored_selforganization_name, 
        cassl_certs=cassl_certs_list, 
        creds=creds
    )

#Set server config
@dns_routes.route('/server-config', methods=['POST'])
@login_required
def server_config():
    set_selfssl_status = 'disable'
    set_cassl_status = 'disable'
    cassl_certs_list = list_cassl_certs()

    listen_port = request.form['listen_port']
    server_name = request.form['server_name']
    enable_self_ssl = request.form.get('enable_ssl') == 'true'
    set_cassl_certs = request.form['set_cassl_certs']
    enable_ca_ssl = request.form.get('enable_cassl') == 'true'
    set_default_dns = request.form.get('set_default_dns') == 'true'

    if enable_self_ssl and not enable_ca_ssl:
        # Check if the self-signed certificate and key files exist
        if (os.path.exists(SSL_CERT_PATH) and os.path.exists(SSL_KEY_PATH)):
            set_selfssl_status = 'enable'
        else:
            return "Create SSL"

    elif enable_ca_ssl and not enable_self_ssl:
        if set_cassl_certs in cassl_certs_list:
            if set_cassl_certs == server_name:    
                set_cassl_status = 'enable'
            elif set_cassl_certs == '':
                return "Create caSSL"
            else:
                set_cassl_status = 'enable'
                server_name = set_cassl_certs
        else:
            return "Create caSSL"

    elif listen_port == '443' and not enable_self_ssl and not set_default_dns:
        if not (os.path.exists(SSL_CERT_PATH) and os.path.exists(SSL_KEY_PATH)):
            return "Create SSL"
        else:
            set_selfssl_status = 'enable'
    
    elif listen_port == '443' and set_default_dns:
        listen_port = '9331'
        set_selfssl_status = 'disable'
        server_name = '_'

    elif set_default_dns:
        listen_port = '9331'
        set_selfssl_status = 'disable'
        server_name = '_'

    else:
        pass

    # Update the Nginx configuration
    update_nginx_config(listen_port, server_name, set_selfssl_status, set_cassl_status)
    
    if listen_port == '80':
        disable_default_ng()
    elif listen_port != '80':
        enable_default_ng()

    reload_nginx()

    if set_selfssl_status == 'enable':
        return "SSL enabled"
    elif set_cassl_status == 'enable':
        return "caSSL enabled"
    elif listen_port == '9331' and set_selfssl_status == 'disable' and server_name == '_':
        return "Set default"
    else:
        return "Server updated"

#Update nginx config
def update_nginx_config(listen_port, server_name, set_selfssl_status, set_cassl_status):
    try:
        with open(NGINX_CONFIG_PATH, 'r') as config_file:
            lines = config_file.readlines()

        if server_name != '_' or '':
            CASSL_CERT_PATH = f'/etc/letsencrypt/live/{server_name}/fullchain.pem'
            CASSL_KEY_PATH = f'/etc/letsencrypt/live/{server_name}/privkey.pem'
        else:
            CASSL_CERT_PATH, CASSL_KEY_PATH = 'None', 'None'

        updated_lines = []
        
        ssl_line = "listen 443 ssl;"
        self_ssl_line1 = f"ssl_certificate {SSL_CERT_PATH};"
        self_ssl_line2 = f"ssl_certificate_key {SSL_KEY_PATH};"
        ca_ssl_line1 = f"ssl_certificate {CASSL_CERT_PATH};"
        ca_ssl_line2 = f"ssl_certificate_key {CASSL_KEY_PATH};"

        updated_listen = False  # Flag to keep track of whether the "listen" line has been updated

        for line in lines:
            if line.strip().startswith("listen "):
                if not updated_listen:  # Only add one "listen" line
                    if set_selfssl_status == 'enable':
                        updated_lines.append(f"    {ssl_line}\n    {self_ssl_line1}\n    {self_ssl_line2}\n")
                    elif set_cassl_status == 'enable':
                        updated_lines.append(f"    {ssl_line}\n    {ca_ssl_line1}\n    {ca_ssl_line2}\n")
                    else:
                        updated_lines.append(f"    listen {listen_port};\n")
                    updated_listen = True
            elif line.strip().startswith("ssl_certificate"):
                if set_selfssl_status != 'enable' or set_cassl_status != 'enable':
                    # Skip these lines if self-signed SSL is not enabled
                    continue
            elif line.strip().startswith("server_name "):
                updated_lines.append(f"    server_name {server_name};\n")
            else:
                updated_lines.append(line)

        with open(NGINX_CONFIG_PATH, 'w') as config_file:
            config_file.writelines(updated_lines)

    except Exception as e:
        return f"Error updating nginx conf: {e}"

#Generate self-signed SSL
@dns_routes.route('/gen-self-signed-ssl', methods=['POST'])
@login_required
def gen_self_signed_ssl():
    common_name = request.form["common_name"]
    validity_days = int(request.form["validity_days"])
    organization_name = request.form["organization_name"]
    self_ssl_result = a2d_self_ssl(common_name, validity_days, organization_name)
    if self_ssl_result == "sSSL generated":
        reload_nginx()
        return "sSSL generated"
    else:
        return "Error generating SSL"

#Generate CN signed SSL
@dns_routes.route('/gen-ca-ssl', methods=['POST'])
@login_required
def gen_ca_ssl():
    ca_common_name = request.form["ca_common_name"]
    email_id = request.form["email_id"]
    cassl_certs_list = list_cassl_certs()

    if ca_common_name not in cassl_certs_list:
        if is_valid_email(email_id):
            ca_ssl = a2d_ca_ssl(ca_common_name, email_id)
            if ca_ssl == "caSSL generated":
                return "caSSL generated"
            else:
                return "Error generating SSL"
        else:
            return "invalid email"
    else:
        return "caSSL exist"

#Email syntax check
def is_valid_email(email):
    pattern = r'^[\w\.-]+@[\w\.-]+\.\w+$'
    if re.match(pattern, email):
        return True
    else:
        return False

#Remove CN signed SSL
@dns_routes.route('/rm-ca-ssl', methods=['POST'])
@login_required
def rm_ca_ssl():
    rm_cassl_certs = request.form['rm_cassl_certs']
    cassl_certs_list = list_cassl_certs()
    current_server_name = read_nginx_config('server_name')

    if rm_cassl_certs in cassl_certs_list:
        if rm_cassl_certs != current_server_name:
            rm_cassl = a2d_rm_cassl(rm_cassl_certs)
            if rm_cassl == "caSSL removed":
                reload_nginx()
                return "caSSL removed"
            else:
                return "Error removing SSL"
        else:
            return "caSSL in use"
    else:
        return "No caSSL"

#Get the list of CA SSL
def list_cassl_certs():
    result = a2d_ca_list()
    return result

#HTML data display
@dns_routes.route('/for-html', methods=['GET'])
@login_required
def for_html():
    # Read the current DNS configuration values
    current_listen_port = read_nginx_config('listen')
    if current_listen_port == 'ssl':
        current_listen_port = '443'
    current_server_name = read_nginx_config('server_name')
    current_SSL_crt = read_nginx_config('ssl_certificate')

    if current_SSL_crt:
        CASSL_CERT_PATH = f'/etc/letsencrypt/live/{current_server_name}/fullchain.pem'
    else:
        CASSL_CERT_PATH = 'None'

    if current_SSL_crt == SSL_CERT_PATH:
        current_ssl_status = 'Self-Signed'
    elif current_SSL_crt == CASSL_CERT_PATH:
        current_ssl_status = 'CA Validated'
    else:
        current_ssl_status = 'Disabled'

    #Read SSL certificate status
    if current_SSL_crt:
        output = read_ssl(current_SSL_crt)
        common_name, organization_name, expiry_date = output.split(',')

        # Handle empty values by replacing them with None
        if common_name == '':
            common_name = 'None'
        if organization_name == '':
            organization_name = 'None'
        if expiry_date == '':
            expiry_date = 'None'

        current_date = datetime.now()
        current_date =current_date.strftime('%Y-%m-%d %H:%M:%S')

        if expiry_date and expiry_date < current_date:
            expiry_date = 'Expired'
    else:
        common_name, organization_name, expiry_date = 'None', 'None', 'None'

    #Read service status
    nginx_status = get_ctl_status('nginx')
    gunicorn_status = get_ctl_status('a2d')

    response_data = {
        "listen_port": current_listen_port,
        "server_name": current_server_name,
        "current_ssl_status": current_ssl_status,
        "common_name": common_name,
        "organization_name": organization_name,
        "expiry_date": expiry_date,
        "nginx_status": nginx_status,
        "gunicorn_status": gunicorn_status
    }
    
    return jsonify(response_data)

#Get systemctl status
def get_ctl_status(ctl_name):
    try:
        output = subprocess.check_output(['systemctl', 'status', ctl_name], text=True)
        ctl_status = re.search(r'since\s(.*)', output).group(1)
        ctl_status = ctl_status.split(";")[1].strip()
        ctl_status = str(ctl_status)
        return ctl_status
        
    except subprocess.CalledProcessError as e:
        ctl_status = str(e)
        return ctl_status



import os
from flask import Blueprint, render_template, request, redirect, session
from a2d.modals.user import verify_pin, create_user, create_user_passphrase, decrypt_user_passphrase
from a2d.modals.creds import save_credentials, get_credentials
from a2d.routes.handle_db import reset_dbs
from functools import wraps
import configparser

auth_routes = Blueprint('auth', __name__)

def handle_digits():
    digit1 = request.form['digit1']
    digit2 = request.form['digit2']
    digit3 = request.form['digit3']
    digit4 = request.form['digit4']
    digit5 = request.form['digit5']
    digit6 = request.form['digit6']
    pin = digit1 + digit2 + digit3 + digit4 + digit5 + digit6
    return pin

@auth_routes.route('/login', methods=['GET', 'POST'])
def login():
    if request.method == 'POST':
        pin = handle_digits()
        if verify_pin(pin):
            session['pin'] = pin
            return redirect('/')
        else:
            return render_template('login.html', error=True)
    return render_template('login.html')

@auth_routes.route('/logout')
def logout():
    session.clear()
    return redirect('/login')

# Check if the user is authenticated before accessing protected routes
def passphrase_required(f):
    @wraps(f)
    def decorated_function(*args, **kwargs):
        if check_passphrase_file() == "user_passphrase.bin does not exist." or passphrase_authenticated():
            return f(*args, **kwargs)
        return render_template('verifypassphrase.html')
    return decorated_function

# Check if the user has entered the correct passphrase
def passphrase_authenticated():
    return session.get('pp_authenticated', False)

@auth_routes.route('/create-passphrase', methods=['GET', 'POST'])
@passphrase_required
def passphrase_change():
    if request.method == 'POST':
        pin = handle_digits()
        create_user(pin)
        session['pin'] = pin
        passphrase = request.form['passphrase']
        create_user_passphrase(passphrase)
        session['passphrase'] = passphrase
        return redirect('/logout')
    return render_template('updatepin.html')

@auth_routes.route('/verify-passphrase', methods=['POST'])
def verify_passphrase():
    passphrase = request.form.get('passphrase')
    stored_passphrase = decrypt_user_passphrase()
    if passphrase == stored_passphrase:
        session['passphrase'] = passphrase
        session['pp_authenticated'] = True
        return redirect('/create-passphrase')
    else:
        return redirect('/logout')

@auth_routes.route('/pin-change', methods=['GET', 'POST'])
def pin_change():
    if request.method == 'POST':
        pin = handle_digits()
        create_user(pin)
        session['pin'] = pin
        return redirect('/logout')
    else:
        if check_passphrase_file() == "user_passphrase.bin does not exist.":
            return redirect('/create-passphrase')
        return render_template('verifypassphrase.html')

@auth_routes.route('/check_pin_file')
def check_pin_file():
    pin_file_exists = os.path.isfile('/etc/a2d/.creds/user_pin.bin')
    if pin_file_exists:
        message = "7965732062696e"
    else:
        message = "6e6f2062696e"
    return message

def check_passphrase_file():
    passphrase_file_exists = os.path.isfile('/etc/a2d/.creds/user_passphrase.bin')
    if passphrase_file_exists:
        message = "user_passphrase.bin exists."
    else:
        message = "user_passphrase.bin does not exist."
    return message

# Define a decorator function to check if the user is logged in
def login_required(f):
    @wraps(f)
    def decorated_function(*args, **kwargs):
        if 'pin' not in session:
            return redirect('/logout')
        return f(*args, **kwargs)
    return decorated_function

@auth_routes.route('/AnD-creds', methods=['POST'])
@login_required
def handle_creds():
    callsign = request.form['callsign']
    txgroup = request.form['txgroup']
    dapnetuser = request.form['dapnetuser']
    dapnetpass = request.form['dapnetpass']
    aprsapi = request.form['aprsapi']

    credentials = get_credentials()
    previous_callsign = credentials.get('callsign')
    previous_txgroup = credentials.get('txgroup')
    previous_dapnetuser = credentials.get('dapnetuser')
    previous_dapnetpass = credentials.get('dapnetpass')

    if previous_callsign != callsign or previous_txgroup != txgroup or previous_dapnetuser != dapnetuser or previous_dapnetpass != dapnetpass:
        reset_dbs()
        # Update the configuration file with the status code
        config = configparser.ConfigParser()
        config.optionxform = lambda option: option
        config.read('/etc/a2d/dapnet_data.ini')
        reset_dapnet_http_code = '201'
        config.set('DAPNETVerify', 'HTTPStatus', reset_dapnet_http_code)   
        with open('/etc/a2d/dapnet_data.ini', 'w') as configfile:
            config.write(configfile)     
        
        runconfig = configparser.ConfigParser()
        runconfig.optionxform = lambda option: option
        runconfig.read('/etc/a2d/runscript_data.ini')
        reset_aprs_check_ini = 'unverified'
        runconfig.set('APRSCheck', 'APRSCheck', reset_aprs_check_ini)
        with open('/etc/a2d/runscript_data.ini', 'w') as runconfigfile:
            runconfig.write(runconfigfile)
    
    save_credentials(callsign, txgroup, dapnetuser, dapnetpass, aprsapi)
    return redirect('/')



from cryptography.fernet import Fernet
import os

encryption_key_file = '/etc/a2d/.keys/app.key'
passphrase_key_file = '/etc/a2d/.keys/passphrase.key'
user_pin_file = '/etc/a2d/.creds/user_pin.bin'
user_passphrase_file = '/etc/a2d/.creds/user_passphrase.bin'
cipher = None

def generate_encryption_key():
    encryption_key = Fernet.generate_key()
    os.makedirs(os.path.dirname(encryption_key_file), exist_ok=True)
    with open(encryption_key_file, 'wb') as key_file:
        key_file.write(encryption_key)
    # Change permissions to 400
    if os.path.exists(encryption_key_file):
        os.chmod(encryption_key_file, 0o400)
    return encryption_key

def get_encryption_key():
    if os.path.isfile(encryption_key_file):
        with open(encryption_key_file, 'rb') as key_file:
            encryption_key = key_file.read()
    else:
        encryption_key = generate_encryption_key()
    return encryption_key

def verify_pin(pin):
    encryption_key = get_encryption_key()
    cipher = Fernet(encryption_key)
    with open(user_pin_file, 'rb') as file:
        encrypted_pin = file.read()
        decrypted_pin = cipher.decrypt(encrypted_pin).decode()
        if pin == decrypted_pin:
            return True
    return False

def create_user(pin):
    encryption_key = get_encryption_key()
    os.makedirs(os.path.dirname(user_pin_file), exist_ok=True)
    cipher = Fernet(encryption_key)
    encrypted_pin = cipher.encrypt(pin.encode())
    with open(user_pin_file, 'wb') as file:
        file.write(encrypted_pin)

def generate_passphrase_key():
    encryption_key = Fernet.generate_key()
    os.makedirs(os.path.dirname(passphrase_key_file), exist_ok=True)
    with open(passphrase_key_file, 'wb') as key_file:
        key_file.write(encryption_key)
    # Change permissions to 400
    if os.path.exists(passphrase_key_file):
        os.chmod(passphrase_key_file, 0o400)
    return encryption_key

def get_passphrase_key():
    if os.path.isfile(passphrase_key_file):
        with open(passphrase_key_file, 'rb') as key_file:
            encryption_key = key_file.read()
    else:
        encryption_key = generate_passphrase_key()
    return encryption_key

def decrypt_user_passphrase():
    encryption_key = get_passphrase_key()
    cipher = Fernet(encryption_key)
    with open(user_passphrase_file, 'rb') as file:
        encrypted_passphrase = file.read()
        decrypted_passphrase = cipher.decrypt(encrypted_passphrase).decode()
    return decrypted_passphrase

def create_user_passphrase(passphrase):
    encryption_key = get_passphrase_key()
    cipher = Fernet(encryption_key)
    encrypted_passphrase = cipher.encrypt(passphrase.encode())
    with open(user_passphrase_file, 'wb') as file:
        file.write(encrypted_passphrase)


import os
import sys
from a2d.a2d_utils.pseudo_aprs_utils import process_aprs_data_parallel
from a2d.a2d_utils.a2d_utils import establish_connection, retrieve_data_from_table, create_table_if_not_exists

# Database directory
data_dir = f'/var/lib/a2d/dbs'

def main():
    # Get the absolute path of the current script
    current_dir = os.path.dirname(os.path.abspath(__file__))

    # Add the 'utils' directory to the Python module search path if not already present
    utils_dir = os.path.join(current_dir, 'utils')
    if utils_dir not in sys.path:
        sys.path.append(utils_dir)

    # Establish connection to the APRS messages database
    aprs_messages_db = f'{data_dir}/pseudo_aprs_messages.db'
    aprs_conn, aprs_cursor = establish_connection(aprs_messages_db)

    # Create the table if it doesn't exist
    create_table_if_not_exists(aprs_cursor, 'pseudo_aprs_messages', '(messageid TEXT PRIMARY KEY, srccall TEXT, message TEXT, trgcall TEXT)')

    # Retrieve data from the 'callssid' table
    callssid_db = f'{data_dir}/callssid.db'
    callssid_conn, callssid_cursor = establish_connection(callssid_db)
    rows = retrieve_data_from_table(callssid_cursor, 'callssid')
    callssid_conn.close()

    # Close the connection to the APRS messages database
    aprs_conn.close()

    # Process APRS messages using multiprocessing
    process_aprs_data_parallel(rows, 'pseudo_aprs_messages')

if __name__ == "__main__":
    main()

# Create a SQLite empty aprs database
aprs_messages_db = f'{data_dir}/aprs_messages.db'
aprs_conn, aprs_cursor = establish_connection(aprs_messages_db)

# Create a table to store the data
create_table_if_not_exists(aprs_cursor, 'aprs_messages', '(messageid TEXT PRIMARY KEY, srccall TEXT, message TEXT, trgcall TEXT)')

# Commit the changes and close the connection
aprs_conn.commit()
aprs_conn.close()



import os
import subprocess
import datetime
import re
from a2d.routes.nginx import reload_nginx, read_nginx_config, disable_default_ng, enable_default_ng

def read_ssl(cert_file):
    if not cert_file or not os.path.exists(cert_file):
        return "None,None,None"

    try:
        # Use subprocess to run openssl commands
        subject = subprocess.check_output(['openssl', 'x509', '-noout', '-subject', '-in', cert_file]).decode()
        expiry_date = subprocess.check_output(['openssl', 'x509', '-noout', '-enddate', '-in', cert_file]).decode()

        common_name = None
        organization_name = None

        # Use regular expressions to extract CN and O fields
        cn_match = re.search(r'CN\s*=\s*([^/,]+)', subject)
        o_match = re.search(r'O\s*=\s*([^/,]+)', subject)

        if cn_match:
            common_name = cn_match.group(1).strip()
        if o_match:
            organization_name = o_match.group(1).strip()

        expiry_date = expiry_date.split('=')[1].strip()
        expiry_date = datetime.datetime.strptime(expiry_date, '%b %d %H:%M:%S %Y %Z')
        expiry_date = expiry_date.strftime('%Y-%m-%d %H:%M:%S')

        return f"{common_name},{organization_name},{expiry_date}"

    except Exception as e:
        return "None,None,None"

def a2d_self_ssl(common_name, validity_days, organization_name):
    ssl_cert_path = '/etc/nginx/ssl/a2d-ssl.crt'
    ssl_key_path = '/etc/nginx/ssl/a2d-ssl.key'
    try:
        # Check if the certificate and key already exist
        if os.path.exists(ssl_cert_path) and os.path.exists(ssl_key_path):
            os.remove(ssl_cert_path)
            os.remove(ssl_key_path)

        # Redirect stdout and stderr to /dev/null to avoid unknown output from openssl
        with open(os.devnull, 'w') as devnull:
            # Generate the self-signed certificate
            subprocess.run([
                "openssl", "req", "-new", "-newkey", "rsa:4096", "-days", str(validity_days),
                "-nodes", "-x509", "-keyout", ssl_key_path, "-out", ssl_cert_path,
                "-subj", f"/CN={common_name}/O={organization_name}"
            ], check=True, stdout=devnull, stderr=devnull)

        # Verify if the certificate and key were successfully generated
        if os.path.exists(ssl_cert_path) and os.path.exists(ssl_key_path):
            return "sSSL generated"
        else:
            return "Error generating SSL"

    except subprocess.CalledProcessError as e:
        return "Error generating SSL"
    except Exception as e:
        return "Error generating SSL"

def a2d_rm_cassl(common_name):
    try:
        # Execute the certbot command to delete the certificate
        subprocess.run(["certbot", "delete", "--cert-name", common_name, "--non-interactive"], check=True)
        return "caSSL removed"
    except subprocess.CalledProcessError as e:
        return "Error removing SSL"

def a2d_ca_ssl(common_name, email):
    disable_default_ng()
    reload_nginx()
    listen_port = read_nginx_config('listen')
    try:
        # Execute the certbot command to generate the certificate
        subprocess.run(["certbot", "certonly", "-d", common_name, "--standalone", "--preferred-challenges", "http", "--email", email, "--agree-tos", "--non-interactive"], check=True)
        if listen_port != '80':
            enable_default_ng()
        reload_nginx()
        return "caSSL generated"
    except subprocess.CalledProcessError as e:
        if listen_port != '80':
            enable_default_ng()
        reload_nginx()
        return "Error generating SSL"

def a2d_ca_list():
    try:
        # Execute the 'ls' and 'grep' commands
        result = subprocess.run(["ls", "/etc/letsencrypt/live/"], stdout=subprocess.PIPE, stderr=subprocess.PIPE, text=True)
        output = result.stdout

        # Filter out the "README" entry
        certificates = [cert.strip() for cert in output.split('\n') if cert.strip() != "README"]
        
        return certificates

    except subprocess.CalledProcessError as e:
        return []

def a2d_rm_selfssl():
    ssl_cert_path = '/etc/nginx/ssl/a2d-ssl.crt'
    ssl_key_path = '/etc/nginx/ssl/a2d-ssl.key'
    # Check if the certificate and key already exist
    if os.path.exists(ssl_cert_path) and os.path.exists(ssl_key_path):
        os.remove(ssl_cert_path)
        os.remove(ssl_key_path)



from requests.auth import HTTPBasicAuth
import requests
from a2d.a2d_utils.decr_usrinfo import retrieve_usrinfo
from a2d.a2d_utils.a2d_utils import establish_connection, retrieve_data_from_table, remove_cronjob, read_ini_data, write_ini_data

# Retrieve values from the retrieve_usrinfo function
data = retrieve_usrinfo()
DAPNET_USER = data['dapnet_user']
DAPNET_PASS = data['dapnet_pass']

#File and Directories
data_dir = f'/var/lib/a2d/dbs'

# Establish connection to the SQLite database
ham_info_db = f'{data_dir}/ham_info.db'
conn, cursor = establish_connection(ham_info_db)
row = retrieve_data_from_table(cursor, 'ham_info')
conn.close()

# DAPNET function
def dapnet(new_msg_str):
    login = DAPNET_USER
    passwd = DAPNET_PASS
    text = new_msg_str
    callsign = [row[0][1]]
    url = 'http://www.hampager.de:8080/calls'
    txgroup = row[0][2]

	# DAPNET push functions
    def push_to_dapnet(text, callsign, login, passwd, url, txgroup):
        payload = {
            "text": text,
            "callSignNames": callsign,
            "transmitterGroupNames": [txgroup],
            "emergency": False
        }
        headers = {
        "Content-Type": "application/json"
        }
        auth = HTTPBasicAuth(login, passwd)
        response = requests.post(url, json=payload, headers=headers, auth=auth)
        return response.status_code

	# DAPNET send
    status_code = push_to_dapnet(text, callsign, login, passwd, url, txgroup)
    
    # Update the configuration file with the status code
    dapnet_data = '/etc/a2d/dapnet_data.ini'
    config = read_ini_data(dapnet_data)
    status_code_str = str(status_code)
    config.set('DAPNETVerify', 'HTTPStatus', status_code_str)
    write_ini_data(config, dapnet_data)
    
    #Stop cron if the status code is not 201
    if status_code_str != '201':
        remove_cronjob()



#coding=utf-8
#!/usr/bin/env python3

import multiprocessing
from multiprocessing import Manager
from a2d.a2d_utils.get_aprs import get_aprs
from a2d.a2d_utils.post_dapnet import dapnet
from a2d.a2d_utils.a2d_utils import establish_connection, retrieve_data_from_table, create_table_if_not_exists

#Database directory
data_dir = f'/var/lib/a2d/dbs'

# Establish connection to the SQLite databases
callssid_db = f'{data_dir}/callssid.db'
callssid_conn, callssid_cursor = establish_connection(callssid_db)
rows = retrieve_data_from_table(callssid_cursor, 'callssid')
callssid_conn.close()

def find_new_msg():
    # Connect to the pseudo_aprs_messages database
    pseudo_aprs_db = f'{data_dir}/pseudo_aprs_messages.db'
    connection, pseudo_cursor = establish_connection(pseudo_aprs_db)
    pseudo_aprs_rows = retrieve_data_from_table(pseudo_cursor, 'pseudo_aprs_messages')
    connection.close()
    pseudo_aprs_set = set(row[0] for row in pseudo_aprs_rows)  # Extract the messageid from each row

    return pseudo_aprs_set

def new_msg(srccall, trgcall, message):
    new_msg_str = srccall + '(' + trgcall + '):' + message
    dapnet(new_msg_str)

def process_row(row, aprs_messages_db, pseudo_aprs_set, lock):
    conn, cursor = establish_connection(aprs_messages_db)

    try:
        # Acquire the lock before accessing the database
        lock.acquire()

        result = get_aprs(row)
        if result is None:
            return

        entries, trgcall = result

        if not entries:
            return

        for item in entries:
            messageid = item['messageid']
            srccall = item['srccall']
            message = item['message']

            # Check if messageid already exists in the SQLite table
            cursor.execute("SELECT messageid FROM aprs_messages WHERE messageid = ?", (messageid,))
            result = cursor.fetchone()

            # If messageid doesn't exist, insert the data into the SQLite table
            if not result:
                cursor.execute("INSERT INTO aprs_messages (messageid, srccall, message, trgcall) VALUES (?, ?, ?, ?)",
                               (messageid, srccall, message, trgcall))

                # Check if the messageid also doesn't exist in the pseudo_aprs_messages
                if messageid not in pseudo_aprs_set:
                    # Pass the new message to new_msg() function
                    new_msg(srccall, trgcall, message)

        conn.commit()
    finally:
        # Release the lock after the database operations are done
        lock.release()

    cursor.close()
    conn.close()

def APRS_messages():
    # Create a SQLite database
    aprs_messages_db = f'{data_dir}/aprs_messages.db'
    conn, cursor = establish_connection(aprs_messages_db)
    create_table_if_not_exists(cursor, 'aprs_messages', '(messageid TEXT PRIMARY KEY, srccall TEXT, message TEXT, trgcall TEXT)')
    cursor.close()
    conn.close()

    # Get pseudo_aprs_set using your find_new_msg() function
    pseudo_aprs_set = find_new_msg()

    # Number of processes to create
    num_processes = multiprocessing.cpu_count()

    # Create a process pool
    pool = multiprocessing.Pool(processes=num_processes)

    # Create a shared lock using Manager
    manager = Manager()
    lock = manager.Lock()

    # Prepare arguments for the process_row function
    args = [(row, aprs_messages_db, pseudo_aprs_set, lock) for row in rows]

    # Map the process_row function to the arguments in the pool
    pool.starmap(process_row, args)

    # Close the pool and wait for all processes to finish
    pool.close()
    pool.join()

if __name__ == "__main__":
    APRS_messages()



import sqlite3
import configparser

#db utils
def establish_connection(db_file):
    # Establish connection to the SQLite database
    conn = sqlite3.connect(db_file)
    cursor = conn.cursor()
    return conn, cursor

def retrieve_data_from_table(cursor, table_name):
    # Retrieve data from the specified table
    cursor.execute(f'SELECT * FROM {table_name}')
    rows = cursor.fetchall()
    return rows

def create_table_if_not_exists(cursor, table_name, columns):
    # Create a table if it doesn't exist in the database
    create_table_query = f'CREATE TABLE IF NOT EXISTS {table_name} {columns}'
    cursor.execute(create_table_query)

def message_id_exists(cursor, table_name, message_id):
    # Check if the message ID exists in the specified table
    cursor.execute(f'SELECT messageid FROM {table_name} WHERE messageid = ?', (message_id,))
    result = cursor.fetchone()
    return bool(result)

#Cron util
def remove_cronjob():
    command = '/usr/bin/python3 -m a2d.runscripts'
    lines = []
    
    try:
        with open('/etc/cron.d/a2d', 'r') as cron_file:
            lines = cron_file.readlines()
    except FileNotFoundError:
        pass
    
    filtered_lines = [line for line in lines if not line.strip().endswith(command)]

    with open('/etc/cron.d/a2d', 'w') as cron_file:
        cron_file.writelines(filtered_lines)

#Ini read/write utils
def read_ini_data(ini_file):
    config = configparser.ConfigParser()
    config.optionxform = lambda option: option
    config.read(ini_file)
    return config

def write_ini_data(config, ini_file):
    with open(ini_file, 'w') as configfile:
        config.write(configfile)



import subprocess
import re
from flask import Blueprint, jsonify
from a2d.routes.auth import login_required

network_routes = Blueprint('network', __name__)

@network_routes.route('/get-rtt', methods=['GET'])
@login_required
def get_rtt():
    hosts = {"aprs": "api.aprs.fi", "dapnet": "www.hampager.de"}
    rtt_results = {}

    for key, host in hosts.items():
        try:
            result = subprocess.run(['ping', '-c', '1', host], capture_output=True, text=True, timeout=5)
            output_lines = result.stdout.splitlines()

            rtt_line = [line for line in output_lines if "time=" in line]

            if rtt_line:
                rtt_match = re.search(r"time=(\d+(\.\d+)?)", rtt_line[0])
                if rtt_match:
                    rtt_value = float(rtt_match.group(1))
                    rtt_results[key + "_rtt"] = f"{rtt_value} ms"
                else:
                    rtt_results[key + "_rtt"] = "Unreachable"
            else:
                rtt_results[key + "_rtt"] = "Unreachable"

        except subprocess.TimeoutExpired:
            rtt_results[key + "_rtt"] = "Unreachable"

    return jsonify(rtt_results)



import os
import hashlib
import yaml
from flask import Blueprint, request, redirect, render_template, Response
from a2d.routes.auth import login_required
from a2d.modals.user import decrypt_user_passphrase
from a2d.modals.creds import get_credentials

config_routes = Blueprint('config', __name__)

file_paths = {
    "00": "/etc/a2d/a2d_adv_conf.ini",
    "01": "/etc/a2d/.creds/a2d_AnDinfo.conf",
    "10": "/etc/a2d/.keys/a2d_AnDinfo.key",
}

@config_routes.route("/export", methods=["POST"])
@login_required
def export_files():
    passphrase = request.form.get("passphrase")
    if not verify_passphrase(passphrase):
        return "Invalid Passphrase", 400

    all_files_exist = True
    for file_path in file_paths.values():
        if not os.path.exists(file_path):
            all_files_exist = False
            break

    if not all_files_exist:
        return "Files missing", 401

    if all_files_exist:
        data = {file_name: read_binary_file(file_path) for file_name, file_path in file_paths.items()}
        watermark = calculate_watermark(data)
        export_data = {"d": data, "f": watermark}
        yaml_data = yaml.dump(export_data)
        sha256 = hashlib.sha256()
        sha256.update(yaml_data.encode('utf-8'))

        response = Response(yaml_data, mimetype="application/x-yaml")
        response.headers["Content-Disposition"] = "attachment; filename=a2d_config_backup.yaml"

        return response

@config_routes.route("/import", methods=["POST"])
@login_required
def import_files():
    passphrase = request.form.get("passphrase")
    if not verify_passphrase(passphrase):
        return "Invalid Passphrase", 400

    import_file = request.files.get("import_file")
    if import_file and import_file.filename.endswith(".yaml"):
        yaml_data = import_file.read().decode('utf-8')
        data = yaml.safe_load(yaml_data)
        watermark = data.get("f")

        if verify_watermark(data["d"], watermark):
            if os.path.exists('/etc/a2d/.keys/a2d_AnDinfo.key'):
                os.chmod('/etc/a2d/.keys/a2d_AnDinfo.key', 0o644)
            for file_name, file_data in data["d"].items():
                if file_name in file_paths:
                    create_directory_if_not_exists(file_paths[file_name])
                    write_binary_file(file_paths[file_name], file_data)
            os.chmod('/etc/a2d/.keys/a2d_AnDinfo.key', 0o400)

            return redirect('/') #This wont work, implemented in client side
        else:
            return "File tampered", 401
    else:
        return "Invalid file", 402

def read_binary_file(file_path):
    with open(file_path, "rb") as file:
        return file.read()

def write_binary_file(file_path, data):
    with open(file_path, "wb") as file:
        file.write(data)

def create_directory_if_not_exists(file_path):
    directory = os.path.dirname(file_path)
    if not os.path.exists(directory):
        os.makedirs(directory)

def calculate_watermark(data):
    yaml_data = yaml.dump(data, default_flow_style=False, allow_unicode=True)
    sha256 = hashlib.sha256()
    sha256.update(yaml_data.encode('utf-8'))    
    return sha256.digest()

def verify_watermark(data, watermark):
    calculated_watermark = calculate_watermark(data)
    return calculated_watermark == watermark

def verify_passphrase(passphrase):
    stored_passphrase = decrypt_user_passphrase()
    return passphrase == stored_passphrase

@config_routes.route('/help-full')
@login_required
def help_full():
    creds = get_credentials
    return render_template('help_full.html', creds=creds)

@config_routes.route('/help-short')
def help_short():
    return render_template('help_short.html')



from flask import Blueprint, jsonify, request, redirect
import os
import sqlite3
import configparser
import datetime
from a2d.modals.creds import get_credentials
from a2d.routes.run import restart_service
from a2d.routes.auth import login_required
from a2d.routes.handle_db import reset_dbs

data_routes = Blueprint('data', __name__)

adv_conf = '/etc/a2d/a2d_adv_conf.ini'
runscript_data = '/etc/a2d/runscript_data.ini'

@data_routes.route('/fetch-logs')
@login_required
def fetch_logs():
    if os.path.exists('/var/lib/a2d/dbs/aprs_messages.db') and os.path.exists('/var/lib/a2d/dbs/pseudo_aprs_messages.db'):
        aprs_db_path = '/var/lib/a2d/dbs/aprs_messages.db'
        pseudo_db_path = '/var/lib/a2d/dbs/pseudo_aprs_messages.db'

        config = configparser.ConfigParser()
        config.optionxform = lambda option: option  # Preserve case sensitivity
        config.read(adv_conf)
        max_logs = config.getint('LOGS', 'NumberOfLogs', fallback='')

        try:
            # Connect to the aprs database
            connection = sqlite3.connect(aprs_db_path)
            cursor = connection.cursor()
            cursor.execute('SELECT * FROM aprs_messages')
            aprs_rows = cursor.fetchall()
            connection.close()

            # Connect to the pseudo_aprs_messages database
            connection = sqlite3.connect(pseudo_db_path)
            cursor = connection.cursor()
            cursor.execute('SELECT * FROM pseudo_aprs_messages')
            pseudo_aprs_rows = cursor.fetchall()
            connection.close()

            # Convert the lists of rows to sets for easy comparison
            aprs_set = set(aprs_rows)
            pseudo_aprs_set = set(pseudo_aprs_rows)

            # Find the difference between the two sets (rows present in aprs_messages but not in pseudo_aprs_messages)
            diff_set = aprs_set - pseudo_aprs_set

            # Sort the logs in descending order based on the first element (row[0])
            logs = []
            count = 0  # Counter to keep track of the number of logs added
            for row in sorted(diff_set, reverse=True, key=lambda x: x[0]):
                log = f'{row[0]} > {row[1]} ({row[3]}): {row[2]}'
                logs.append(log)
                count += 1
                if count >= max_logs:  # Limit the number of logs user setting
                    break

            return jsonify(logs)

        except sqlite3.OperationalError:
            pass

    return jsonify([])  # Return an empty list if an error occurs

@data_routes.route('/save-adv-conf', methods=['POST'])
@login_required
def save_adv_conf():
    config = configparser.ConfigParser()
    config.optionxform = lambda option: option  # Preserve case sensitivity
    config.read(adv_conf)
    previous_ssids = config.get('SSID', 'PreferredSSIDs', fallback='').split(',')

    # Update the values in the config object
    intervalmin = request.form['intervalmin']
    intervalh = request.form['intervalh']
    ssids = ','.join(request.form.getlist('ssids[]'))
    ssids_list = ssids.split(',')
    logcounts = request.form['logcounts']

    config['LOGS']['NumberOfLogs'] = logcounts

    if intervalmin == '0' and intervalh == '0':
        config['Timer']['IntervalMinutes'] = '15'
        config['Timer']['IntervalHours'] = '0'
    elif intervalmin == '1' and intervalh == '0':
        config['Timer']['IntervalMinutes'] = '2'
        config['Timer']['IntervalHours'] = '0'
    else:
        config['Timer']['IntervalMinutes'] = intervalmin
        config['Timer']['IntervalHours'] = intervalh
    
    if previous_ssids != ssids_list:
        if ssids is None:
            config['SSID']['PreferredSSIDs'] = '0'
        else:
            config['SSID']['PreferredSSIDs'] = ssids
        reset_dbs()

    with open(adv_conf, 'w') as config_file:
        config.write(config_file, space_around_delimiters=False)

    restart_service()
    return redirect('/')

def get_adv_conf_values():
    config = configparser.ConfigParser()
    config.optionxform = lambda option: option  # Preserve case sensitivity
    config.read(adv_conf)

    intervalmin = config.get('Timer', 'IntervalMinutes', fallback='')
    intervalh = config.get('Timer', 'IntervalHours', fallback='')
    ssids = config.get('SSID', 'PreferredSSIDs', fallback='').split(',')
    logcounts = config.get('LOGS', 'NumberOfLogs', fallback='')

    return intervalmin, intervalh, ssids, logcounts


@data_routes.route('/for-dash')
@login_required
def for_dash():
    credentials = get_credentials()
    #Get callsign and callsign2
    callsign = credentials.get('callsign', 'Setup Configuration')
    callsign2 = credentials.get('callsign')
    
    #ssids-display
    config = configparser.ConfigParser()
    config.read(adv_conf)
    ssids_display = config.get('SSID', 'PreferredSSIDs', fallback='').split(',')
    ssids_display = ['-{}'.format(ssid) for ssid in ssids_display if ssid.strip()]
    ssids_display = ' '.join(ssids_display)

    #Run interval
    intervalmin = config.get('Timer', 'IntervalMinutes', fallback='')
    intervalh = config.get('Timer', 'IntervalHours', fallback='')
    run_interval = f"{intervalh}h {intervalmin}min"
    
    #Last run
    config.read(runscript_data)
    # Get the values for last run unix time
    last_run_unix = config.get('InternalTimeStamp', 'LastRunTime', fallback='')
    last_run_human = 'None' if not last_run_unix else datetime.datetime.fromtimestamp(float(last_run_unix)).strftime('%Y-%m-%d %H:%M:%S')

    dash_data = {
        "callsign": callsign,
        "callsign2": callsign2,
        "ssids_display": ssids_display,
        "run_interval": run_interval,
        "last_run_human": last_run_human
    }

    return jsonify(dash_data)



from a2d.a2d_utils.a2d_utils import message_id_exists, establish_connection
from a2d.a2d_utils.get_aprs import get_aprs
import multiprocessing

def process_aprs_data(rows, table_name):
    # SQLite database
    aprs_messages_db = f'/var/lib/a2d/dbs/{table_name}.db'
    conn, cursor = establish_connection(aprs_messages_db)

    data_to_insert = []

    for c in rows:
        result = get_aprs(c)
        if result is None:
            continue

        entries, trgcall = result

        if not entries:
            continue

        for item in entries:
            messageid = item['messageid']
            srccall = item['srccall']
            message = item['message']

            # Check if messageid already exists in the SQLite table
            if not message_id_exists(cursor, table_name, messageid):
                data_to_insert.append((messageid, srccall, message, trgcall))

    # Close the connection to the database before creating the table
    conn.close()

    # Re-establish connection to the database to create the table and insert data
    conn, cursor = establish_connection(aprs_messages_db)

    # Batch insert the data into the SQLite table
    cursor.executemany(f'INSERT INTO {table_name} VALUES (?, ?, ?, ?)', data_to_insert)

    # Commit the changes and close the connection
    conn.commit()
    conn.close()

def process_aprs_data_parallel(rows, table_name):
    num_processes = max(1, min(multiprocessing.cpu_count(), len(rows)))
    chunk_size = max(1, len(rows) // num_processes)
    chunks = [rows[i:i + chunk_size] for i in range(0, len(rows), chunk_size)]

    processes = []
    for chunk in chunks:
        process = multiprocessing.Process(target=process_aprs_data, args=(chunk, table_name))
        processes.append(process)
        process.start()

    for process in processes:
        process.join()



from a2d.a2d_utils.decr_usrinfo import retrieve_usrinfo
import requests

data = retrieve_usrinfo()
APRSAPI_KEY = data['aprsapi_key']
callsign_nossid = data['callsign_nossid']

def get_aprs(c):
    if c[2] == '0':
        url = "https://api.aprs.fi/api/get?what=msg&dst=" + c[1] + "&apikey="+APRSAPI_KEY+"&format=json"
        trgcall = c[2]
    else:
        url = "https://api.aprs.fi/api/get?what=msg&dst=" + c[1] + "-" + c[2] + "&apikey="+APRSAPI_KEY+"&format=json"
        trgcall = c[2]

    aprs = requests.get(url)
    aprs = aprs.json()

    try: #If ARPS server returns "Get failed, query ratelimit" (because of too frequent requests)
        entries = aprs['entries']
    except KeyError:
        return None

    return entries, trgcall

def aprs_check():
    url = "https://api.aprs.fi/api/get?what=msg&dst="+callsign_nossid+"&apikey="+APRSAPI_KEY+"&format=json"

    try:
        aprs_check = requests.get(url)
        entries_check = aprs_check.json()
    except KeyError:
        return None

    return entries_check



import os
from flask import Flask, render_template, request, session, redirect, jsonify, send_file
from a2d.routes.auth import auth_routes
from a2d.routes.run import run_routes
from a2d.routes.dns import dns_routes
from a2d.routes.system import system_routes
from a2d.routes.network import network_routes
from a2d.routes.config import config_routes
from a2d.routes.reset import reset_routes
from a2d.routes.data import data_routes, get_adv_conf_values
from a2d.modals.creds import get_credentials

app = Flask(__name__)
app.config['APP_VERSION'] = '2.0.3'

#Storing session key implemented to avoid nginx error
SESSION_KEY_FILE = "/etc/a2d/.keys/session_key.bin"
SESSION_KEY_DIR = os.path.dirname(SESSION_KEY_FILE)

def generate_and_store_session_key():
    if not os.path.exists(SESSION_KEY_DIR):
        try:
            os.makedirs(SESSION_KEY_DIR)
        except OSError as e:
            pass
    if os.path.exists(SESSION_KEY_DIR):
        session_key = os.urandom(32)
        with open(SESSION_KEY_FILE, "wb") as key_file:
            key_file.write(session_key)
    else:
        pass

def load_session_key():
    try:
        with open(SESSION_KEY_FILE, "rb") as key_file:
            return key_file.read()
    except FileNotFoundError:
        return None

def set_session_key():
    session_key = load_session_key()
    if session_key is None:
        generate_and_store_session_key()
        session_key = load_session_key()
    return session_key

app.secret_key = set_session_key()

# Register the auth_routes blueprint
app.register_blueprint(auth_routes)
app.register_blueprint(run_routes)
app.register_blueprint(config_routes)
app.register_blueprint(data_routes)
app.register_blueprint(dns_routes)
app.register_blueprint(system_routes)
app.register_blueprint(network_routes)
app.register_blueprint(reset_routes)

@app.route('/')
def home():
    if 'pin' in session:
        creds = get_credentials()
        active = 'Dashboard'
        service_status = request.args.get('service_status')
        last_run_human = request.args.get('last_run_human')
        run_interval = request.args.get('run_interval')
        callsign = request.args.get('callsign')
        callsign2 = request.args.get('callsign2')
        all_files_exist = request.args.get('allFilesExist')
        intervalmin, intervalh, ssids, logcounts = get_adv_conf_values()
        return render_template(
            'index.html', 
            creds=creds, 
            active=active, 
            intervalmin=intervalmin, 
            intervalh=intervalh, 
            ssids=ssids, 
            logcounts=logcounts, 
            service_status=service_status, 
            last_run_human=last_run_human, 
            run_interval=run_interval, 
            callsign=callsign, 
            callsign2=callsign2, 
            all_files_exist=all_files_exist
        )
    else:
        return redirect('/login')

@app.route('/version', methods=['GET'])
def get_version():
    return jsonify({'version': app.config['APP_VERSION']})

@app.route('/image')
def get_image():
    image_path = '/usr/share/images/logo_a2d.png'
    return send_file(image_path, as_attachment=True)

if __name__ == '__main__':
    app.run(host='127.0.0.1', port=9333)



import os
import configparser
from cryptography.fernet import Fernet
import io

encryption_key = '/etc/a2d/.keys/a2d_AnDinfo.key'
a2d_config_file = '/etc/a2d/.creds/a2d_AnDinfo.conf'

def generate_encryption_key():
    if not os.path.exists(encryption_key):
        key = Fernet.generate_key()
        with open(encryption_key, 'wb') as key_file:
            key_file.write(key)
        # Change permissions to 400
        os.chmod(encryption_key, 0o400)
    return encryption_key

def encrypt_config(config):
    generate_encryption_key()  # Ensure encryption key file exists
    with open(encryption_key, 'rb') as keyfile:
        key = keyfile.read()
    cipher = Fernet(key)
    config_buffer = io.StringIO()
    config.write(config_buffer)
    serialized_config = config_buffer.getvalue().encode()
    encrypted_config = cipher.encrypt(serialized_config)
    return encrypted_config

def decrypt_config(config_path):
    with open(config_path, 'rb') as configfile:
        encrypted_config = configfile.read()
    with open(encryption_key, 'rb') as keyfile:
        key = keyfile.read()
    cipher = Fernet(key)
    decrypted_config = cipher.decrypt(encrypted_config)
    config = configparser.ConfigParser()
    config.read_string(decrypted_config.decode())
    return config

def save_credentials(callsign, txgroup, dapnetuser, dapnetpass, aprsapi):
    config = configparser.ConfigParser()
    config['DEFAULT'] = {
        'callsign_nossid': callsign,
        'txgroup': txgroup,
        'DAPNET_USER': dapnetuser,
        'DAPNET_PASS': dapnetpass,
        'APRSAPI_KEY': aprsapi
    }
    encrypted_config = encrypt_config(config)
    with open(a2d_config_file, 'wb') as configfile:
        configfile.write(encrypted_config)

def get_credentials():
    config_path = a2d_config_file
    if os.path.isfile(config_path):
        config = decrypt_config(config_path)
        callsign = config.get('DEFAULT', 'callsign_nossid', fallback='')
        txgroup = config.get('DEFAULT', 'txgroup', fallback='')
        dapnetuser = config.get('DEFAULT', 'DAPNET_USER', fallback='')
        dapnetpass = config.get('DEFAULT', 'DAPNET_PASS', fallback='')
        return {
            'callsign': callsign,
            'txgroup': txgroup,
            'dapnetuser': dapnetuser,
            'dapnetpass': dapnetpass,
        }
    else:
        return {'message': 'Input credentials.'}



import os
import subprocess
import configparser
from flask import Blueprint, redirect, jsonify
from a2d.routes.auth import login_required
from a2d.a2d_utils.a2d_utils import remove_cronjob

run_routes = Blueprint('run', __name__)

def generate_cron_job(interval_minutes, interval_hours, interval_hours_uni):
    cron_jobs = []
    if interval_minutes == 0 and interval_hours == 0:
        cron_jobs.append(f"*/15 * * * * root /usr/bin/python3 -m a2d.runscripts")
    elif interval_minutes == 1 and interval_hours == 0:
        cron_jobs.append(f"*/2 * * * * root /usr/bin/python3 -m a2d.runscripts")
    elif interval_minutes != 0 and interval_hours == 0:
        cron_jobs.append(f"*/{interval_minutes} * * * * root /usr/bin/python3 -m a2d.runscripts")
    elif interval_minutes == 0 and interval_hours != 0:
        cron_jobs.append(f"0 */{interval_hours} * * * root /usr/bin/python3 -m a2d.runscripts")    
    elif interval_minutes != 0 and interval_minutes != 1 and interval_hours != 0 and interval_hours != 1:
        cron_jobs.append(f"*/{interval_minutes} */{interval_hours} * * * root /usr/bin/python3 -m a2d.runscripts")
    else:
        cron_jobs.append(f"0 */{interval_hours_uni} * * * root /usr/bin/python3 -m a2d.runscripts")
        cron_jobs.append(f"0-59/{interval_minutes} * * * * root /usr/bin/python3 -m a2d.runscripts")
    return cron_jobs

def add_cronjob():
    # Read values from the configuration file
    config_file = '/etc/a2d/a2d_adv_conf.ini'
    config = configparser.ConfigParser()
    config.read(config_file)

    # Get the values for IntervalMinutes and IntervalHours
    interval_minutes = int(config.get('Timer', 'IntervalMinutes'))
    interval_hours = int(config.get('Timer', 'IntervalHours'))
    interval_hours_uni = interval_hours + 1

    # Generate the cron job schedule using the updated function
    cron_schedule_list = generate_cron_job(interval_minutes, interval_hours, interval_hours_uni)

    # Create and write the cron job to /etc/cron.d/a2d.cron
    with open('/etc/cron.d/a2d', 'a') as cron_file:
        for cron_schedule in cron_schedule_list:
            cron_file.write(f'{cron_schedule}\n')

def check_cronjob():
    # Find the cron job that matches the specified command
    command = '/usr/bin/python3 -m a2d.runscripts'
    runstatus = "Stopped"  # Set a default value before the loop
    try:
        with open('/etc/cron.d/a2d', 'r') as cron_file:
            for line in cron_file:
                if line.strip().endswith(command):
                    runstatus = "Running"
                    break  # No need to continue reading the file once a match is found
    except FileNotFoundError:
        pass  # Handle the case when the file doesn't exist

    return runstatus

@run_routes.route('/start-service')
@login_required
def start_service():
    if os.path.isfile('/etc/a2d/.creds/a2d_AnDinfo.conf'):
        runstatus = check_cronjob()
        if runstatus == "Stopped":
            # Call the function to add the cron job
            add_cronjob()
            run_command = ["nohup", "/usr/share/scripts/run_a2d.sh", "&"]
            subprocess.Popen(run_command)
        return redirect('/')
    return redirect('/')

@run_routes.route('/stop-service')
@login_required
def stop_service():
    runstatus = check_cronjob()
    if runstatus == "Running":
        # Call the function to remove the cron job
        remove_cronjob()
    return redirect('/')

def restart_service():
    runstatus = check_cronjob()
    if runstatus == "Running":
        remove_cronjob()
        add_cronjob()

@run_routes.route('/service-status')
@login_required
def service_status():
    runconfig_file = '/etc/a2d/runscript_data.ini'
    runconfig = configparser.ConfigParser()
    runconfig.read(runconfig_file)
    aprs_server = runconfig.get('APRSKeyVerify', 'apiKeyStatus', fallback='')
    
    dapconfig = configparser.ConfigParser()
    dapconfig_file = '/etc/a2d/dapnet_data.ini'
    dapconfig.read(dapconfig_file)
    dapnet_server = dapconfig.get('DAPNETVerify', 'HTTPStatus', fallback='')

    status_messages = {
        ('invalid', '201'): 'Invalid APRS API key',
        ('wrong', '201'): 'Wrong APRS API key',
        ('valid', '401'): 'Wrong DAPNET user or passwd',
        ('valid', '400'): 'Wrong callsign or txgroup',
        ('invalid', '401'): 'Invalid APRS API key/Wrong DAPNET user or passwd',
        ('wrong', '401'): 'Wrong APRS API key/Wrong DAPNET user or passwd',
        ('invalid', '400'): 'Invalid APRS API key/Wrong callsign or txgroup',
        ('wrong', '400'): 'Wrong APRS API key/Wrong callsign or txgroup',
    }

    if aprs_server == 'valid' and dapnet_server == '201':
        service_status = {
            "color_status": True,
            "status_message": check_cronjob()
        }
    
    elif aprs_server == 'valid' and dapnet_server not in ('201', '401', '400'):
        service_status = {
            "color_status": False,
            "status_message": "DAPNET HTTP" + dapnet_server
        }
    else:
        service_status = {
            "color_status": False,
            "status_message": status_messages.get((aprs_server, dapnet_server))
        }

    return jsonify({"service_status": service_status})



from cryptography.fernet import Fernet

def retrieve_usrinfo():
    key_file = '/etc/a2d/.keys/a2d_AnDinfo.key'
    encrypted_file = '/etc/a2d/.creds/a2d_AnDinfo.conf'

    with open(key_file, 'rb') as file:
        key = file.read()

    cipher_suite = Fernet(key)

    with open(encrypted_file, 'rb') as file:
        encrypted_data = file.read()

    decrypted_data = cipher_suite.decrypt(encrypted_data).decode('utf-8')

    # Parse the decrypted data as needed
    data_dict = {}
    for line in decrypted_data.split('\n'):
        line = line.strip()
        if line and ' = ' in line:
            key, value = line.split(' = ', 1)
            data_dict[key] = value

    return data_dict



#coding=utf-8
#!/usr/bin/env python3

bind = '127.0.0.1:9333'
workers = 3 #For 2 core
threads = 2 #Per worker optimum for 2 core and 512MB memory
max_requests = 1000 #Restart worker after 1000 requests
timeout = 30
max_requests_jitter = 100 #Restart connection after 100 req
errorlog = '/var/log/a2d_gu_error.log'
loglevel = 'error'
logrotate = 1000000 #rotate after 1MB



import psutil
from flask import Blueprint, jsonify
from a2d.routes.auth import login_required

system_routes = Blueprint('system', __name__)

@system_routes.route('/system-info', methods=['GET'])
@login_required
def system_info():
    style = 'style="color: #d4660c; font-weight: bold;"'

    #CPU temperature
    raw_cpu_temperature = get_cpu_temperature()
    if raw_cpu_temperature is not None:
        cpu_temperature_value = float(raw_cpu_temperature)
        formatted_cpu_temperature = f" {cpu_temperature_value:.2f}°C"

        if cpu_temperature_value > 80:
            temperature_style = style
        else:
            temperature_style = ''
        
        cpu_temperature = f'<span {temperature_style}>{formatted_cpu_temperature}</span>'
    else:
        cpu_temperature = "Not available"

    #System memory
    raw_memory_usage = get_system_memory_usage()
    if raw_memory_usage is not None:
        memory_usage_value = float(raw_memory_usage)
        memory_usage = f" {memory_usage_value}%"
    
        if memory_usage_value > 80:
            memory_style = style
        else:
            memory_style = ''
        
        system_memory_usage = f'<span {memory_style}>{memory_usage}</span>'
    else:
        system_memory_usage = "Not available"
    
    #CPU load
    cpu_load = get_cpu_load()
    cpu_load = f" {cpu_load}%"

    system_data = {
            "cpu_temperature": cpu_temperature,
            "system_memory_usage": system_memory_usage,
            "cpu_load": cpu_load
        }

    return jsonify(system_data)

def get_cpu_temperature():
    try:
        with open('/sys/class/thermal/thermal_zone0/temp', 'r') as temp_file:
            temp_str = temp_file.read().strip()
            temp_milli_celsius = int(temp_str)
            temp_celsius = temp_milli_celsius / 1000.0
            return temp_celsius
    except Exception as e:
        print("An error occurred:", e)
    
    return None

def get_system_memory_usage():
    memory = psutil.virtual_memory()
    return memory.percent

def get_cpu_load():
    load = psutil.cpu_percent(interval=1)
    return load



import os
import sqlite3

def reset_dbs():
    db_folder = '/var/lib/a2d/dbs'
    aprs_db_path = os.path.join(db_folder, 'aprs_messages.db')
    pseudo_aprs_db_path = os.path.join(db_folder, 'pseudo_aprs_messages.db')

    if os.path.exists(aprs_db_path) and os.path.exists(pseudo_aprs_db_path):
        clear_table(aprs_db_path, 'aprs_messages')
        clear_table(pseudo_aprs_db_path, 'pseudo_aprs_messages')
    elif os.path.exists(aprs_db_path):
        clear_table(aprs_db_path, 'aprs_messages')
    elif os.path.exists(pseudo_aprs_db_path):
        clear_table(pseudo_aprs_db_path, 'pseudo_aprs_messages')

def clear_table(db_path, table_name):
    try:
        conn = sqlite3.connect(db_path)
        cursor = conn.cursor()
        
        cursor.execute(f'DELETE FROM {table_name}')

        conn.commit()
        conn.close()

        os.remove(db_path)

    except Exception as e:
        pass
